# litds

<!-- WARNING: THIS FILE WAS AUTOGENERATED! DO NOT EDIT! -->

## Developer Guide

### Setup

``` sh
# create conda environment
$ mamba env create -f env.yml

# update conda environment
$ mamba env update -n litds --file env.yml
```

### Install

``` sh
pip install -e .

# install from pypi
pip install litds
```

### nbdev

``` sh
# activate conda environment
$ conda activate litds

# make sure the litds package is installed in development mode
$ pip install -e .

# make changes under nbs/ directory
# ...

# compile to have changes apply to the litds package
$ nbdev_prepare
```

### Publishing

``` sh
# publish to pypi
$ nbdev_pypi

# publish to conda
$ nbdev_conda --build_args '-c conda-forge'
```

# Usage

## Installation

Install latest from the GitHub
[repository](https://github.com/dsm-72/litds):

``` sh
$ pip install git+https://github.com/dsm-72/litds.git
```

or from [conda](https://anaconda.org/dsm-72/litds)

``` sh
$ conda install -c dsm-72 litds
```

or from [pypi](https://pypi.org/project/litds/)

``` sh
$ pip install litds
```

## Documentation

Documentation can be found hosted on GitHub
[repository](https://github.com/dsm-72/litds)
[pages](https://dsm-72.github.io/litds/). Additionally you can find
package manager specific guidelines on
[conda](https://anaconda.org/dsm-72/litds) and
[pypi](https://pypi.org/project/litds/) respectively.

# Brief Datasets Demo

``` python
dd = DiamondsDataset()
dd.getone()
```

    (tensor([[ 0.0491,  0.0015],
             [ 0.1555,  0.3419],
             [-0.3727,  0.3598],
             [ 0.6811,  0.3734],
             [ 0.4492, -0.7379]]),
     tensor([0., 1., 2., 3., 4.]))

``` python
dd.plot(palette='mako_r')
```

![](index_files/figure-commonmark/cell-3-output-1.png)

``` python
od = OrbitsDataset()
od.getone()
```

    (tensor([[ 0.0203, -0.2043],
             [-0.3889, -0.1781],
             [ 0.0290,  0.6299],
             [-0.3386,  0.8402],
             [ 0.9571,  0.0337]]),
     tensor([0., 1., 2., 3., 4.]))

``` python
od.plot(palette='mako_r')
```

![](index_files/figure-commonmark/cell-5-output-1.png)

``` python
eb = EmbryoidBodies2018DataModule(
    primary='pca',
    batch_size=8,
)
```

``` python
eb.setup()
```

    EB Loader:   0%|          | 0/126 [00:00<?, ?it/s]

    Loading data
    Data ready!

``` python
dl = eb.train_dataloader()
```

``` python
eb.train_ds.df[eb.train_ds.df.columns[-5:]].head()
```

<div>
<style scoped>
    .dataframe tbody tr th:only-of-type {
        vertical-align: middle;
    }
&#10;    .dataframe tbody tr th {
        vertical-align: top;
    }
&#10;    .dataframe thead th {
        text-align: right;
    }
</style>

|       | d97       | d98       | d99       | d100      | samples   |
|-------|-----------|-----------|-----------|-----------|-----------|
| 4854  | -0.633847 | -0.701649 | -1.466198 | -0.381950 | Day 06-09 |
| 6121  | 0.432789  | 1.089373  | -0.968371 | -0.211686 | Day 06-09 |
| 7620  | 0.042552  | 0.870117  | -1.264836 | -1.664296 | Day 12-15 |
| 12560 | 1.770667  | -0.594136 | 1.893129  | -1.064998 | Day 18-21 |
| 10336 | -2.947320 | -1.072601 | 1.554807  | 0.170369  | Day 18-21 |

</div>
